/*
 * PilotC.cpp
 *
 * Created: 17.09.2012 19:20:03
 *  Author: ossenbrink
 */ 

#include "Arduino.h"
#include "PilotC.h"
#include "Timer.h"

#define STX			0x02
#define ETX			0x03
#define ENQ			0x05
#define ACK			0x06
#define NAK			0x15
#define DC4			0x14

#define NONE				0
#define	MODE_MANUAL_ACK		1
#define MODE_PC_ACK			2
#define CONNECT_ACK			3

#define PC_MODE				1
#define MANUAL_MODE			2

Timer t;

PilotC::PilotC()
{
	mode = MANUAL_MODE;
	Serial1.begin(19200, SERIAL_7E1);
	inputString.reserve(200);
}

void PilotC::connect()
{
	sendCommand("DC");
	checkInput(CONNECT_ACK, 1000);
	t.every(7000, toggleMode);			// Toggle mode every 10s to avoid alarms   -> Problem with function pointer
}

uint8_t PilotC::sendCommand(String cmd)
{
	String checksumStr = "";
	
	if (ackwait)
	return -1;

	ackwait = true;								// Don't send commands if previous ack not received
	
	uint8_t checksum = calcChecksum(cmd);
	checksumStr = String(checksum, HEX);
	checksumStr.toUpperCase();
	
	Serial1.write(STX);
	Serial1.print(cmd);
	Serial1.print(checksumStr);
	Serial1.write(ETX);
	Serial.print(millis());
	Serial.print("[TX]:");
	//Serial.write(STX);
	Serial.print(cmd);
	Serial.print(checksumStr);
	//Serial.write(ETX);
	Serial.write('\n');
	return 0;
}

int PilotC::bolus(int flowrate, int amount)
{
	//sprint(str, "0x%x", i);
	String tmpStr = String(flowrate, HEX);
	sendCommand("PB;199A;0010"); //Bolus;Flowrate;ml
}

void PilotC::clearInput()
{
	awaiting = NULL;
	inputString = "";
}


void PilotC::update()
{
	t.update();
}

uint8_t PilotC::calcChecksum(String str)
{
	uint8_t checksum = 0;

	for (uint8_t i = 0; i < str.length(); i++)
	{
		checksum += str.charAt(i);
	}
	//checksum &= 0xFF;
	checksum = ~checksum;
	checksum &= 0xFF;

	return checksum;
}

void PilotC::checkInput(int ack, int timeout)
{
	awaiting = ack;
	timerIndex = t.after(timeout, PilotC::clearInput);
}



void PilotC::setModePC()
{
	if (connected)
	{
		Serial.print(millis());
		sendCommand("MO;2"); //1 : manual, 2 : PC control
		checkInput(MODE_PC_ACK, 1000);
	}
}

static void PilotC::toggleMode ()
{
	if (connected)
	{
		if (mode != MANUAL_MODE)
		{
			sendCommand("MO;1"); //1 : manual, 2 : PC control
			//checkInput(MODE_CHANGE_ACK, 200);
			t.after(200, setModePC);
		}
		else
		setModePC();
	}
}

void PilotC::inputHandler()
{
	switch (awaiting)
	{
		case MODE_PC_ACK:
		Serial.println(inputString);
		if (inputString.equals("CBC"))			// everything ok
		{
			mode = PC_MODE;
			inputString = "";
			t.stop(timerIndex);
			Serial.println(":-)");
		}
		else if (inputString.equals("I;0615"))	// already mode in PC_MODE
		{
			mode = PC_MODE;
			Serial.println(":-(");
		}
		else if (inputString.equals("I;1E"))	// disconnected
		{
			mode = NONE;
			connected = false;
			connect();
		}
		break;
		case MODE_MANUAL_ACK:
		Serial.println(inputString);
		if (inputString.indexOf("C"))
		{
			mode = MANUAL_MODE;
			inputString = "";
			t.stop(timerIndex);
			Serial.println(":-)");
		}
		else if (inputString.indexOf("I;0615"))
		{
			mode = MANUAL_MODE;
			Serial.println(":-(");
		}
		else if (inputString.indexOf("I;1E"))
		{
			mode = NONE;
			connected = false;
			connect();
		}
		break;
		case CONNECT_ACK:
		Serial.println(inputString);
		if (inputString.equals("C;015124/15810521X36"))
		{
			connected = true;
			inputString = "";
			t.stop(timerIndex);
			Serial.println("Connected");
		}
		else
		connected = false;
		break;
		case NULL:
		inputString = "";
		
	}
}

/*
  SerialEvent occurs whenever a new data comes in the
 hardware serial RX.  This routine is run between each
 time loop() runs, so using delay inside loop can delay
 response.  Multiple bytes of data may be available.
 */
void PilotC::serialEvent1() 
{
  while (Serial1.available()) 
  {
    // get the new byte:
    char inChar = (char)Serial1.read();
	//Serial.write(inChar);
	
	// if char is for line detection protocol just send the [DC4] ack.
	switch (inChar)
	{
		case ENQ:
			//Serial.println("[RX]: ENQ");
			Serial1.write(DC4);
			break;
		case STX:
			Serial.print(millis());
			Serial.println("[RX]: STX");
			if (rx_active) break;
			rx_active = true;
			break;
		case ETX:
			Serial.print(millis());
			Serial.println("[RX]: ETX");	
			Serial1.write(ACK);
			inputHandler();
			rx_active = false;
			break;
		case ACK:
			Serial.print(millis());
			Serial.println("[RX]: ACK");
			ackwait = false;
			break;
		case NAK:
			Serial.print(millis());
			Serial.println("[RX]: NAK");
			ackwait = false;
			break;
		default:
			Serial.print("[RX]: ");
			Serial.println(inChar);
			if (rx_active)
			{
				inputString	+= inChar;
			}
			break;	
	}
  }
}